------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--            O C A R I N A . C H E C K E R _ T E S T S U I T E             --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                 Copyright (C) 2007, GET-Telecom Paris.                   --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

with Ocarina.Expander;
with Ocarina.Expander.Queries;
with Ocarina.Checker.Queries;
with Ocarina.Nutils;
with Namet;

with Ocarina.Checker.Queries.Subcomponent_Predicates;
with Ocarina.Checker.Queries.Bound_Predicates;

--  Only for debugging purposes
with Ocarina.Debug; use Ocarina.Debug;

package body Ocarina.Checker_Testsuite is
   use Ocarina.Expander;
   use Ocarina.Expander.Queries;
   use Ocarina.Checker.Queries;
   use Namet;
   use Ocarina.Nutils;

   package OCBP renames
     Ocarina.Checker.Queries.Bound_Predicates.Bound_Query;
   package OCSP renames
     Ocarina.Checker.Queries.Subcomponent_Predicates.Subcomponent_Query;

   procedure Instanciate_Ocarina_Tree
     (Distributed_Application_Root : Node_Id;
      Instance_Root                : out Node_Id);

   ------------------------------
   -- Instanciate_Ocarina_Tree --
   ------------------------------

   procedure Instanciate_Ocarina_Tree
     (Distributed_Application_Root : Node_Id;
      Instance_Root                : out Node_Id)
   is
   begin
      --  Instantiation of the AADL tree

      Instance_Root := Expand_Model (Distributed_Application_Root);

      --  Abort if the instantiation failed

      if No (Instance_Root) then
         W_Line ("The AADL model cannot be expanded.");
         raise Program_Error;
      end if;
   end Instanciate_Ocarina_Tree;

   --------------------------
   -- Perform_All_Checking --
   --------------------------

   procedure Perform_All_Checking
     (Distributed_Application_Root : Node_Id;
      Options                      : Checker_Options)
   is
      Instance_Root : Node_Id;
   begin
      Instanciate_Ocarina_Tree
        (Distributed_Application_Root,
         Instance_Root);

      if Options.Check_Mutexes then
         Do_Check_Mutexes (Instance_Root);
      end if;

   end Perform_All_Checking;

   ----------------------
   -- Do_Check_Mutexes --
   ----------------------

   procedure Do_Check_Mutexes (Instance_Root : Node_Id)
   is
      pragma Unreferenced (Instance_Root);

      Property_Name : constant String := "max_mutexes_number";
      R1            : Result_Set;
      R2            : Result_Set;
      R3            : Result_Set;
      Result        : Result_Set;
      Result_2      : Result_Set;
      Final_Result  : Result_Set;
   begin
      W_Str ("Checking mutexes number... ");
      R1 := Get_Instances_Of_Component_Type (C_Data);
      R2 := Get_Instances_Of_Component_Type (C_Process);
      R3 := Get_Instances_Of_Component_Type (C_Processor);

      for I in 1 .. Cardinal (R3) loop

         Final_Result := Empty_Set;

         --  1/ We search all processes bound to a given processor
         --  instance
         Result :=
           OCBP.Get_Instances_Verifying_Predicate (R2, Get (R3, I));

         for J in 1 .. Cardinal (Result) loop

            --  2/ We search all data which are subcomponent of a
            --  given process instance
            Result_2 :=
              OCSP.Get_Instances_Verifying_Predicate
              (R1, Get (Result, J), PSO_Direct);

            Final_Result := Union (Final_Result, Result_2);

         end loop;

         if Is_Defined_Integer_Property
           (Get (R3, I), Property_Name) and then
           Int (Get_Integer_Property (Get (R3, I), Property_Name)) <
           Int (Cardinal (Final_Result)) then

            W_Line ("Failed!");

            W_Str ("The processor ");
            Write_Name (Compute_Full_Name_Of_Instance (Get (R3, I)));
            W_Line ("manages to many mutexes.");

            raise Program_Error;
         end if;

      end loop;

      W_Line ("Done.");

   end Do_Check_Mutexes;

end Ocarina.Checker_Testsuite;
