------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--                  OCARINA.GENERATORS.PO_HI_ADA.MAPPING                    --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--               Copyright (C) 2006-2007, GET-Telecom Paris.                --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

with Namet; use Namet;
with Utils; use Utils;

with Ocarina.Nodes;
with Ocarina.Nutils;
with Ocarina.Generators.Utils;
with Ocarina.Generators.Messages;
with Ocarina.Generators.Ada_Tree.Nodes;
with Ocarina.Generators.Ada_Tree.Nutils;
with Ocarina.Generators.PO_HI_Ada.Runtime;
with Ocarina.Generators.Ada_Values;

package body Ocarina.Generators.PO_HI_Ada.Mapping is

   use Ocarina.Nodes;
   use Ocarina.Generators.Utils;
   use Ocarina.Generators.Messages;
   use Ocarina.Generators.Ada_Tree.Nodes;
   use Ocarina.Generators.Ada_Tree.Nutils;
   use Ocarina.Generators.PO_HI_Ada.Runtime;
   use Ocarina.Generators.Ada_Values;

   package AAN renames Ocarina.Nodes;
   package AAU renames Ocarina.Nutils;
   package ADN renames Ocarina.Generators.Ada_Tree.Nodes;
   package ADU renames Ocarina.Generators.Ada_Tree.Nutils;

   ---------------------------
   -- Bind_AADL_To_Activity --
   ---------------------------

   procedure Bind_AADL_To_Activity (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Activity_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Activity;

   -----------------------
   -- Bind_AADL_To_CSRA --
   -----------------------

   procedure Bind_AADL_To_CSRA (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Csra_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_CSRA;

   -----------------------------
   -- Bind_AADL_To_Deployment --
   -----------------------------

   procedure Bind_AADL_To_Deployment (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Deployment_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Deployment;

   -----------------------------
   -- Bind_AADL_To_Enumerator --
   -----------------------------

   procedure Bind_AADL_To_Enumerator (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Enumerator_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Enumerator;

   ---------------------------
   -- Bind_AADL_To_Handlers --
   ---------------------------

   procedure Bind_AADL_To_Handlers (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Handlers_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Handlers;

   ----------------------------
   -- Bind_AADL_To_Internals --
   ----------------------------

   procedure Bind_AADL_To_Internals (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Internals_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Internals;

   ----------------------
   -- Bind_AADL_To_Job --
   ----------------------

   procedure Bind_AADL_To_Job (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Job_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Job;

   -----------------------
   -- Bind_AADL_To_Main --
   -----------------------

   procedure Bind_AADL_To_Main (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Main_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Main;

   -----------------------------------
   -- Bind_AADL_To_Marshall_Request --
   -----------------------------------

   procedure Bind_AADL_To_Marshall_Request (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Marshall_Request_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Marshall_Request;

   ------------------------------
   -- Bind_AADL_To_Marshallers --
   ------------------------------

   procedure Bind_AADL_To_Marshallers (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Marshallers_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Marshallers;

   -------------------------
   -- Bind_AADL_To_Naming --
   -------------------------

   procedure Bind_AADL_To_Naming (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Naming_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Naming;

   -------------------------
   -- Bind_AADL_To_Object --
   -------------------------

   procedure Bind_AADL_To_Object (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Object_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Object;

   ----------------------------------
   -- Bind_AADL_To_Process_Request --
   ----------------------------------

   procedure Bind_AADL_To_Process_Request (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Process_Request_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Process_Request;

   -------------------------------
   -- Bind_AADL_To_Request_Type --
   -------------------------------

   procedure Bind_AADL_To_Request_Type (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Request_Type_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Request_Type;

   -----------------------
   -- Bind_AADL_To_SSRA --
   -----------------------

   procedure Bind_AADL_To_SSRA (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Ssra_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_SSRA;

   -----------------------
   -- Bind_AADL_To_Stub --
   -----------------------

   procedure Bind_AADL_To_Stub (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Stub_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Stub;

   -------------------------------------
   -- Bind_AADL_To_Feature_Subprogram --
   -------------------------------------

   procedure Bind_AADL_To_Feature_Subprogram (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Feature_Subprogram_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Feature_Subprogram;

   -----------------------------
   -- Bind_AADL_To_Subprogram --
   -----------------------------

   procedure Bind_AADL_To_Subprogram (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Subprogram_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Subprogram;

   ------------------------------
   -- Bind_AADL_To_Subprograms --
   ------------------------------

   procedure Bind_AADL_To_Subprograms (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Subprograms_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Subprograms;

   ------------------------------
   -- Bind_AADL_To_System_Time --
   ------------------------------

   procedure Bind_AADL_To_System_Time (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_System_Time_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_System_Time;

   ----------------------------------
   -- Bind_AADL_To_Type_Definition --
   ----------------------------------

   procedure Bind_AADL_To_Type_Definition (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Type_Definition_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Type_Definition;

   --------------------------------
   -- Bind_AADL_To_Default_Value --
   --------------------------------

   procedure Bind_AADL_To_Default_Value (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Default_Value_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Default_Value;

   ------------------------
   -- Bind_AADL_To_Types --
   ------------------------

   procedure Bind_AADL_To_Types (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Types_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Types;

   -------------------------------------
   -- Bind_AADL_To_Unmarshall_Request --
   -------------------------------------

   procedure Bind_AADL_To_Unmarshall_Request (G : Node_Id; A : Node_Id) is
      N : Node_Id;
   begin
      N := AAN.Backend_Node (G);

      if No (N) then
         N := New_Node (ADN.K_HI_Tree_Bindings);
         AAN.Set_Backend_Node (G, N);
      end if;

      ADN.Set_Unmarshall_Request_Node (N, A);
      ADN.Set_Frontend_Node (A, G);
   end Bind_AADL_To_Unmarshall_Request;

   ---------------------------------
   -- Map_Distributed_Application --
   ---------------------------------

   function Map_Distributed_Application (E : Node_Id) return Node_Id is
      D : constant Node_Id := New_Node (ADN.K_HI_Distributed_Application);
   begin
      pragma Assert (Is_System (E));

      --  Update the global variable to be able to fetch the root of
      --  the distributed application and generate the source files.

      HI_Distributed_Application_Root := D;

      ADN.Set_Name (D, To_Ada_Name
                    (AAN.Name
                     (AAN.Identifier
                      (E))));
      ADN.Set_Units (D, New_List (ADN.K_List_Id));
      ADN.Set_HI_Nodes (D, New_List (ADN.K_List_Id));

      return D;
   end Map_Distributed_Application;

   -----------------
   -- Map_HI_Node --
   -----------------

   function Map_HI_Node (E : Node_Id) return Node_Id is
      N : constant Node_Id := New_Node (ADN.K_HI_Node);
   begin
      pragma Assert (Is_Process (E));

      --  The name of the node is not the name of the process
      --  component instance, but the name of the process subcomponent
      --  corresponding to this instance.

      ADN.Set_Name (N, To_Ada_Name
                    (AAN.Name
                     (AAN.Identifier
                      (AAN.Parent_Subcomponent
                       (E)))));

      Set_Units (N, New_List (K_List_Id));

      --  Append the partition N to the node list of the PolyORB-HI
      --  distributed application. We are sure that the top of the
      --  entity stack contains the Ada distrubuted application node.

      Append_Node_To_List (N, HI_Nodes (Current_Entity));
      Set_Distributed_Application (N, Current_Entity);

      return N;
   end Map_HI_Node;

   -----------------
   -- Map_HI_Unit --
   -----------------

   function Map_HI_Unit (E : Node_Id) return Node_Id is
      U        : Node_Id;
      L        : List_Id;
      N        : Node_Id;
      P        : Node_Id;
      Ada_Name : Name_Id;
   begin
      pragma Assert (Is_System (E) or else Is_Process (E));

      U := New_Node (ADN.K_HI_Unit, AAN.Identifier (E));
      L := New_List (K_Packages);
      Set_Packages (U, L);

      --  Packages that are common to all nodes

      if Is_System (E) then
         Ada_Name := To_Ada_Name (AAN.Display_Name (AAN.Identifier (E)));

         --  The 'Internal' package

         Get_Name_String (Ada_Name);
         Add_Str_To_Name_Buffer ("_Internals");
         N := Make_Defining_Identifier (Name_Find);
         P := Make_Package_Declaration (N);
         Set_Distributed_Application_Unit (P, U);
         Set_Internals_Package (U, P);
         Append_Node_To_List (P, L);
         Bind_AADL_To_Internals (Identifier (E), P);

         --  The 'CSRA' package

         Get_Name_String (Ada_Name);
         Add_Str_To_Name_Buffer ("_CSRA");
         N := Make_Defining_Identifier (Name_Find);
         P := Make_Package_Declaration (N);
         Set_Distributed_Application_Unit (P, U);
         Set_Csra_Package (U, P);
         Append_Node_To_List (P, L);
         Bind_AADL_To_CSRA (Identifier (E), P);

         --  The 'SSRA' package

         Get_Name_String (Ada_Name);
         Add_Str_To_Name_Buffer ("_SSRA");
         N := Make_Defining_Identifier (Name_Find);
         P := Make_Package_Declaration (N);
         Set_Distributed_Application_Unit (P, U);
         Set_Ssra_Package (U, P);
         Append_Node_To_List (P, L);
         Bind_AADL_To_SSRA (Identifier (E), P);

         --  The 'Marshallers' package

         N := Defining_Identifier (RU (RU_Marshallers, False));
         P := Make_Package_Declaration (N);
         Set_File_Name (P, Add_Suffix_To_Name
                        ("_marshallers", To_Lower (Ada_Name)));
         Set_Has_Custom_File_Name (P, True);
         Set_Distributed_Application_Unit (P, U);
         Set_Marshallers_Package (U, P);
         Append_Node_To_List (P, L);
         Bind_AADL_To_Marshallers (Identifier (E), P);

         --  The 'System_Time' package

         N := Defining_Identifier (RU (RU_System_Time, False));
         P := Make_Package_Declaration (N);
         Set_Distributed_Application_Unit (P, U);
         Set_System_Time_Package (U, P);
         Append_Node_To_List (P, L);
         Bind_AADL_To_System_Time (Identifier (E), P);
      end if;

      --  Packages that are specific to each node

      if Is_Process (E) then
         Ada_Name := To_Ada_Name
           (AAN.Display_Name
            (AAN.Identifier
             (AAN.Parent_Subcomponent
              (E))));

         --  The 'Naming' package

         N := Defining_Identifier (RU (RU_Naming, False));
         P := Make_Package_Declaration (N);
         Set_File_Name (P, Add_Suffix_To_Name
                        ("_naming", To_Lower (Ada_Name)));
         Set_Has_Custom_File_Name (P, True);
         Set_Distributed_Application_Unit (P, U);
         Set_Naming_Package (U, P);
         Append_Node_To_List (P, L);
         Bind_AADL_To_Naming (Identifier (E), P);

         --  The 'Deployment' package

         N := Defining_Identifier (RU (RU_Deployment, False));
         P := Make_Package_Declaration (N);
         Set_File_Name (P, Add_Suffix_To_Name
                        ("_deployment",  To_Lower (Ada_Name)));
         Set_Has_Custom_File_Name (P, True);
         Set_Distributed_Application_Unit (P, U);
         Set_Deployment_Package (U, P);
         Append_Node_To_List (P, L);
         Bind_AADL_To_Deployment (Identifier (E), P);

         --  The 'Types' package

         N := Defining_Identifier (RU (RU_Types, False));
         P := Make_Package_Declaration (N);
         Set_File_Name (P, Add_Suffix_To_Name
                        ("_types",  To_Lower (Ada_Name)));
         Set_Has_Custom_File_Name (P, True);
         Set_Distributed_Application_Unit (P, U);
         Set_Types_Package (U, P);
         Append_Node_To_List (P, L);
         Bind_AADL_To_Types (Identifier (E), P);

         --  The 'Subprograms' package

         N := Defining_Identifier (RU (RU_Subprograms, False));
         P := Make_Package_Declaration (N);
         Set_File_Name (P, Add_Suffix_To_Name
                        ("_subprograms", To_Lower (Ada_Name)));
         Set_Has_Custom_File_Name (P, True);
         Set_Distributed_Application_Unit (P, U);
         Set_Subprograms_Package (U, P);
         Append_Node_To_List (P, L);
         Bind_AADL_To_Subprograms (Identifier (E), P);

         --  The 'Handlers' package

         Get_Name_String (Ada_Name);
         Add_Str_To_Name_Buffer ("_Handlers");
         N := Make_Defining_Identifier (Name_Find);
         P := Make_Package_Declaration (N);
         Set_Distributed_Application_Unit (P, U);
         Set_Handlers_Package (U, P);
         Append_Node_To_List (P, L);
         Bind_AADL_To_Handlers (Identifier (E), P);

         --  The 'Activity' package

         Get_Name_String (Ada_Name);
         Add_Str_To_Name_Buffer ("_Activity");
         N := Make_Defining_Identifier (Name_Find);
         P := Make_Package_Declaration (N);
         Set_Distributed_Application_Unit (P, U);
         Set_Activity_Package (U, P);
         Append_Node_To_List (P, L);
         Bind_AADL_To_Activity (Identifier (E), P);

         --  Main suprogram

         P := Make_Main_Subprogram_Implementation
           (Make_Defining_Identifier (Ada_Name));
         Set_Distributed_Application_Unit (P, U);
         Set_Main_Subprogram (U, P);
         Append_Node_To_List (P, L);
         Bind_AADL_To_Main (Identifier (E), P);
      end if;

      --  Append the Unit to the units list of the current Ada
      --  partition.

      Append_Node_To_List (U, Units (Current_Entity));
      ADN.Set_Entity (U, Current_Entity);

      return U;
   end Map_HI_Unit;

   ------------------
   -- Map_Ada_Time --
   ------------------

   function Map_Ada_Time (T : Time_Type) return Node_Id is
      Time : Unsigned_Long_Long;
      S    : Node_Id;
   begin
      case T.U is
         when Picosecond =>
            --  If we can convert it into nanosecond, we are
            --  OK. Otherwise this is an error because Ada.Real_Time
            --  does not support picoseconds

            if T.T mod 1000 = 0 then
               Time := T.T / 1000;
               S := RE (RE_Nanoseconds);
            else
               return No_Node;
            end if;

         when Nanosecond =>
            Time := T.T;
            S := RE (RE_Nanoseconds);

         when Microsecond =>
            Time := T.T;
            S := RE (RE_Microseconds);

         when Millisecond =>
            Time := T.T;
            S := RE (RE_Milliseconds);

         when Second =>
            Time := T.T;
            S := RE (RE_Seconds);

         when Minute =>
            Time := T.T;
            S := RE (RE_Minutes);

         when Hour =>
            --  Convert it into minutes

            Time := T.T * 60;
            S := RE (RE_Minutes);
      end case;

      return Make_Subprogram_Call
        (S, Make_List_Id (Make_Literal (New_Integer_Value (Time, 1, 10))));
   end Map_Ada_Time;

   ----------------------
   -- Map_Ada_Priority --
   ----------------------

   function Map_Ada_Priority (P : Unsigned_Long_Long) return Node_Id is
      AADL_Delta : constant Unsigned_Long_Long :=
        Get_Max_Priority - Get_Min_Priority;
      System_PF : constant Node_Id := Make_Attribute_Designator
        (RE (RE_Priority), A_First);
      System_PL : constant Node_Id := Make_Attribute_Designator
        (RE (RE_Priority), A_Last);

      Result : Node_Id;
   begin
      --  -------------------------------------------------------
      --  ^              ^                                      ^
      --  Min_AADL       X_AADL                                 Max_AADL

      --  -------------------------------------------------------
      --  ^              ^                                      ^
      --  Min_Ada        X_Ada                                  Max_Ada

      --  X_Ada = Min_Ada + (X_AADL - Min_AADL) * (Max_Ada - Min_Ada)
      --                    ________________________________________
      --                              (Max_AADL - Min_AADL)

      --  (X_AADL - Min_AADL)

      Result := Make_Expression
        (Make_Literal (New_Integer_Value (P, 1, 10)),
         Op_Minus,
         Make_Literal (New_Integer_Value (Get_Min_Priority, 1, 10)));

      --  ...  * (Max_Ada - Min_Ada)

      Result := Make_Expression
        (Result,
         Op_Asterisk,
         Make_Expression (System_PL, Op_Minus, System_PF));

      --  ... / (Max_AADL - Min_AADL)

      Result := Make_Expression
        (Result,
         Op_Slash,
         Make_Literal (New_Integer_Value (AADL_Delta, 1, 10)));

      --  Min_Ada + ...

      Result := Make_Expression (System_PF, Op_Plus, Result);

      return Result;
   end Map_Ada_Priority;

   -----------------------------
   -- Map_Task_Job_Identifier --
   -----------------------------

   function Map_Task_Job_Identifier (E : Node_Id) return Node_Id is
   begin
      Get_Name_String (To_Ada_Name (AAN.Display_Name (Identifier (E))));
      Add_Str_To_Name_Buffer ("_Job");
      return Make_Defining_Identifier (Name_Find);
   end Map_Task_Job_Identifier;

   --------------------------------
   -- Map_Cyclic_Task_Identifier --
   --------------------------------

   function Map_Cyclic_Task_Identifier (E : Node_Id) return Node_Id is
   begin
      Get_Name_String (To_Ada_Name (AAN.Display_Name (Identifier (E))));
      Add_Str_To_Name_Buffer ("_Cyclic_Task");
      return Make_Defining_Identifier (Name_Find);
   end Map_Cyclic_Task_Identifier;

   ----------------------------
   -- Map_Handler_Identifier --
   ----------------------------

   function Map_Handler_Identifier (E : Node_Id) return Node_Id is
   begin
      Get_Name_String (To_Ada_Name (AAN.Display_Name (Identifier (E))));
      Add_Str_To_Name_Buffer ("_Handler");
      return Make_Defining_Identifier (Name_Find);
   end Map_Handler_Identifier;

   -------------------------
   -- Map_Stub_Identifier --
   -------------------------

   function Map_Stub_Identifier (E : Node_Id) return Node_Id is
   begin
      Get_Name_String (To_Ada_Name (AAN.Display_Name (Identifier (E))));
      Add_Str_To_Name_Buffer ("_Stub");
      return Make_Defining_Identifier (Name_Find);
   end Map_Stub_Identifier;

   ------------------------------
   -- Map_Node_Name_Identifier --
   ------------------------------

   function Map_Node_Name_Identifier (E : Node_Id) return Node_Id is
   begin
      pragma Assert (Kind (E) = K_Subcomponent_Instance);

      Get_Name_String (To_Ada_Name (Display_Name (Identifier (E))));
      Add_Str_To_Name_Buffer ("_Node_Name");
      return Make_Defining_Identifier (Name_Find);
   end Map_Node_Name_Identifier;

   -----------------------------
   -- Call_Remote_Subprograms --
   -----------------------------

   procedure Call_Remote_Subprograms
     (Caller_Thread    : Node_Id;
      Spg_Call         : Node_Id;
      Declarations     : List_Id;
      Statements       : List_Id)
   is
      procedure Check_Connection_Consistency (F : Node_Id);
      --  Verifies that the feature F is connected to at least one
      --  destination.

      procedure Call_Remote_Subprograms
        (Root_F         : Node_Id;
         Intermediate_F : Node_Id);
      --  This recursive procedure will unwind recursively all the
      --  destinations of the Root_F feature until it finds a
      --  subprogram. If the remote subprogram is found, add a call
      --  to the subprogram call table (if necessary) and add a
      --  parameter association corresponding to Root_F the the
      --  call to this subprogram. At the top level call to
      --  Call_Remote_Subprograms, Intermediate_F = Root_F.

      procedure Update_Remote_Calls
        (Remote_Spg  : Node_Id;
         Param_Value : Node_Id;
         Param_Name  : Node_Id);
      --  If the call to Spg has not been added yet, create
      --  it. Adds a parameter association Param_Name =>
      --  Param_Value to the call profile.

      Call_List : constant List_Id := New_List (ADN.K_Statement_List);
      --  List that contains the calls to all remote subprogram
      --  connected to Spg.

      Msg_Declared : Boolean := False;

      ----------------------------------
      -- Check_Connection_Consistency --
      ----------------------------------

      procedure Check_Connection_Consistency (F : Node_Id) is
      begin
         if AAU.Length (Destinations (F)) = 0 then
            Display_Located_Error
              (AAN.Loc (F),
               "This feature does not have any destinations",
               Fatal => True);
         end if;
      end Check_Connection_Consistency;

      -----------------------------
      -- Call_Remote_Subprograms --
      -----------------------------

      procedure Call_Remote_Subprograms
        (Root_F         : Node_Id;
         Intermediate_F : Node_Id)
      is
         Destination_F : Node_Id;
         C             : Node_Id;
      begin
         --  Root F has to be a parameter instance

         pragma Assert (Kind (Root_F) = K_Parameter_Instance);

         --  The container of the subprogram containing F has to be
         --  necessarily a Thread.

         pragma Assert (Is_Thread
                        (Parent_Component
                         (Parent_Sequence
                          (Parent_Subcomponent
                           (Parent_Component
                            (Root_F))))));

         --  Check the feature consistency

         Check_Connection_Consistency (Intermediate_F);

         Destination_F := AAN.First_Node (Destinations (Intermediate_F));

         while Present (Destination_F) loop
            C := Parent_Component (Item (Destination_F));

            --  If C is also the parent component of Root_F (which
            --  is necessarily a subprogram). Then display an error
            --  and exit.

            if C = Parent_Component (Root_F) then
               Display_Located_Error
                 (AAN.Loc (C),
                  "This subprogram is involved in a connection cycle",
                  Fatal => True);
            end if;

            if Is_Subprogram (C) then
               --  If C is a subprogram, then append it to the call
               --  list (if necessary, and add a parameter
               --  association corresponding to Root_F and
               --  Destination_F.

               Update_Remote_Calls (C, Root_F, Item (Destination_F));
            else
               --  Otherwise, keep unwinding the destinations

               Call_Remote_Subprograms (Root_F, Item (Destination_F));
            end if;

            Destination_F := AAN.Next_Node (Destination_F);
         end loop;
      end Call_Remote_Subprograms;

      -------------------------
      -- Update_Remote_Calls --
      -------------------------

      procedure Update_Remote_Calls
        (Remote_Spg  : Node_Id;
         Param_Value : Node_Id;
         Param_Name  : Node_Id)
      is
         R_Name  : Name_Id;
         N       : Node_Id;
         Profile : List_Id;
      begin
         --  Check whether a call corresponding to Spg has already
         --  been added to Call_List (we use name table infos
         --  instead of looping on Call list, which is a very fast
         --  way.

         Set_Nat_To_Name_Buffer (Nat (Remote_Spg));
         Add_Str_To_Name_Buffer ("%RemoteCall%");
         Add_Nat_To_Name_Buffer (Nat (Spg_Call));
         R_Name := Name_Find;

         if Get_Name_Table_Info (R_Name) = 0 then
            N := Message_Comment ("Call stub ");
            Append_Node_To_List (N, Statements);

            Profile := New_List (ADN.K_List_Id);

            Add_With_Package
              (RU (RU_Deployment, False),
               Used => True);

            N := Make_Parameter_Association
              (Make_Defining_Identifier (PN (P_From)),
               Make_Defining_Identifier
               (Map_Ada_Enumerator_Name
                (Parent_Subcomponent
                 (Caller_Thread))));
            Append_Node_To_List (N, Profile);

            N := Make_Parameter_Association
              (Make_Defining_Identifier (PN (P_To)),
               Make_Defining_Identifier
               (Map_Ada_Enumerator_Name
                (Parent_Subcomponent
                 (Get_Container_Thread
                  (Remote_Spg)))));
            Append_Node_To_List (N, Profile);

            --  Declare the Message variable if it has not benn
            --  declared yet.

            if not Msg_Declared then
               N := Make_Object_Declaration
                 (Defining_Identifier =>
                    Make_Defining_Identifier
                    (PN (P_Message)),
                  Object_Definition   => RE (RE_Message_Type),
                  Aliased_Present     => True);
               Append_Node_To_List (N, Declarations);

               Msg_Declared := True;
            end if;

            N := Make_Parameter_Association
              (Make_Defining_Identifier (PN (P_Message)),
               Make_Designator (PN (P_Message)));
            Append_Node_To_List (N, Profile);

            --  Create the subprogram call

            N := Make_Subprogram_Call
              (Extract_Designator
               (ADN.Stub_Node
                (Backend_Node
                 (Identifier
                  (Remote_Spg)))),
               Profile);
            Append_Node_To_List (N, Statements);

            --  Mark the call as being added. The info we associate
            --  to the name is the value of the profile list to be
            --  able to get it to add parameter associations.

            Set_Name_Table_Info (R_Name, Nat (Profile));
         else
            Profile := List_Id (Get_Name_Table_Info (R_Name));
         end if;

         --  Create the parameter association and append it the the
         --  call profile.

         N := Make_Parameter_Association
           (Map_Ada_Defining_Identifier (Param_Name),
            Make_Defining_Identifier
            (Map_Ada_Full_Parameter_Name (Spg_Call, Param_Value)));
         Append_Node_To_List (N, Profile);
      end Update_Remote_Calls;

      Spg              : constant Node_Id := Corresponding_Instance (Spg_Call);
      F                : Node_Id;
   begin
      pragma Assert (Is_Thread (Caller_Thread));

      --  The lists have to be created

      if Declarations = No_List or else Statements = No_List then
         raise Program_Error with "Lists have to be created before any "
           & "call to Get_Remote_Subprogram";
      end if;

      if not AAU.Is_Empty (Features (Spg)) then
         F := AAN.First_Node (Features (Spg));

         while Present (F) loop
            if Kind (F) = K_Parameter_Instance and then Is_Out (F) then
               --  Call all the remote subprograms connected the
               --  feature F.

               Call_Remote_Subprograms (F, F);
            end if;

            F := AAN.Next_Node (F);
         end loop;
      end if;

      --  Append the calls to the Statements list

      if not ADU.Is_Empty (Call_List) then
         Append_Node_To_List (ADN.First_Node (Call_List), Statements);

         --  Add use clause to 'Deployment' for more redability

         Add_With_Package (RU (RU_Deployment, False), Used => True);
      end if;
   end Call_Remote_Subprograms;

end Ocarina.Generators.PO_HI_Ada.Mapping;
