------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--                OCARINA.GENERATORS.PO_HI_ADA.SYSTEM_TIME                  --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--               Copyright (C) 2006-2007, GET-Telecom Paris.                --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

with Ocarina.Nodes;
with Ocarina.Entities.Components;

with Ocarina.Generators.Ada_Values;
with Ocarina.Generators.Ada_Tree.Nutils;
with Ocarina.Generators.Ada_Tree.Nodes;
with Ocarina.Generators.PO_HI_Ada.Runtime;

package body Ocarina.Generators.PO_HI_Ada.System_Time is

   use Ocarina.Nodes;
   use Ocarina.Entities.Components;
   use Ocarina.Generators.Ada_Values;
   use Ocarina.Generators.Ada_Tree.Nutils;
   use Ocarina.Generators.PO_HI_Ada.Runtime;

   package ADN renames Ocarina.Generators.Ada_Tree.Nodes;

   ------------------
   -- Package_Spec --
   ------------------

   package body Package_Spec is

      procedure Visit_Architecture_Instance (E : Node_Id);
      procedure Visit_Component_Instance (E : Node_Id);
      procedure Visit_System_Instance (E : Node_Id);

      -----------
      -- Visit --
      -----------

      procedure Visit (E : Node_Id) is
      begin
         case Kind (E) is
            when K_Architecture_Instance =>
               Visit_Architecture_Instance (E);

            when K_Component_Instance =>
               Visit_Component_Instance (E);

            when others =>
               null;
         end case;
      end Visit;

      ---------------------------------
      -- Visit_Architecture_Instance --
      ---------------------------------

      procedure Visit_Architecture_Instance (E : Node_Id) is
      begin
         Visit (Root_System (E));
      end Visit_Architecture_Instance;

      ------------------------------
      -- Visit_Component_Instance --
      ------------------------------

      procedure Visit_Component_Instance (E : Node_Id) is
         Cathegory : constant Component_Category
           := Get_Category_Of_Component (E);
      begin
         case Cathegory is
            when CC_System =>
               Visit_System_Instance (E);

            when others =>
               null;
         end case;
      end Visit_Component_Instance;

      ---------------------------
      -- Visit_System_Instance --
      ---------------------------

      procedure Visit_System_Instance (E : Node_Id) is
         N   : Node_Id;
         U   : constant Node_Id := ADN.Distributed_Application_Unit
           (ADN.Internals_Node (Backend_Node (Identifier (E))));
         Exp : Node_Id;
      begin
         Push_Entity (HI_Distributed_Application_Root);
         Push_Entity (U);
         Set_System_Time_Spec;

         N := Make_Used_Type (RE (RE_Time));
         Append_Node_To_List (N, ADN.Visible_Part (Current_Package));

         Exp := Make_Subprogram_Call
           (RE (RE_Milliseconds),
            Make_List_Id (Make_Literal (New_Integer_Value (1_000, 1, 10))));
         Exp := Make_Expression
           (Make_Subprogram_Call (RE (RE_Clock)),
            Op_Plus,
            Exp);

         N := Make_Object_Declaration
           (Defining_Identifier => Make_Defining_Identifier
              (PN (P_System_Start_Time)),
            Constant_Present    => True,
            Object_Definition   => RE (RE_Time),
            Expression          => Exp);
         Append_Node_To_List (N, ADN.Visible_Part (Current_Package));

         Pop_Entity; --  U
         Pop_Entity; --  HI_Distributed_Application_Root
      end Visit_System_Instance;

   end Package_Spec;

end Ocarina.Generators.PO_HI_Ada.System_Time;
