------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--                  OCARINA.GENERATORS.PO_HI_C.DEPLOYMENT                   --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                 Copyright (C) 2007, GET-Telecom Paris.                   --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

with Namet;

with Ocarina.Nodes;
with Ocarina.Nutils;
with Ocarina.Entities.Components;

with Ocarina.Generators.Utils;
with Ocarina.Generators.C_Values;
with Ocarina.Generators.C_Tree.Nutils;
with Ocarina.Generators.C_Tree.Nodes;
with Ocarina.Generators.PO_HI_C.Mapping;
with Ocarina.Generators.PO_HI_C.Runtime;

package body Ocarina.Generators.PO_HI_C.Deployment is

   use Namet;
   use Ocarina.Nodes;
   use Ocarina.Entities.Components;
   use Ocarina.Generators.Utils;
   use Ocarina.Generators.C_Values;
   use Ocarina.Generators.C_Tree.Nutils;
   use Ocarina.Generators.PO_HI_C.Mapping;
   use Ocarina.Generators.PO_HI_C.Runtime;

   package AAU renames Ocarina.Nutils;
   package CTN renames Ocarina.Generators.C_Tree.Nodes;

   -----------------
   -- Header_File --
   -----------------

   package body Header_File is

      procedure Visit_Architecture_Instance (E : Node_Id);
      procedure Visit_Component_Instance (E : Node_Id);
      procedure Visit_System_Instance (E : Node_Id);
      procedure Visit_Process_Instance (E : Node_Id);
      procedure Visit_Thread_Instance (E : Node_Id);

      Nb_Tasks : Unsigned_Long_Long;

      Last_Node_Enum_Pos   : Nat := 1;
      Last_Thread_Enum_Pos : Nat := 1;
      --  Global variables for the representation clauses of the
      --  generated enumerator types. They point to the next free
      --  position in the corresponding enumaration type. We start by
      --  1 because 0 is the value to indicate that no position has
      --  been assigned to the enumerator yet.

      function Get_Node_Enum_Pos (Name : Name_Id) return Nat;
      function Get_Thread_Enum_Pos (Name : Name_Id) return Nat;
      --  Return the position corresponding to the enumerator. If no
      --  position has been computed for the given enumerator, compute
      --  a new one.

      type Get_Enum_Pos_Access is access function (Name : Name_Id) return Nat;
      procedure Insert_Node_In_List
        (N            : Node_Id;
         L            : List_Id;
         Get_Enum_Pos : Get_Enum_Pos_Access);

      ------------------
      -- Global Lists --
      ------------------

      --  The lists below are global and initialized at the beginning
      --  of visit of each process instance. This does not cause a
      --  consistency nor a concurrency proble since the visits of
      --  processes are done sequentially and the visit of one process
      --  is performed entirely before beginning the visit of another
      --  one.

      Node_Enumerator_List : List_Id;
      --  Global list for all the distributed application nodes
      --  enumeration type.

      Node_Enumerator_Pos_List : List_Id;
      --  Global list to store the representation clause of the node
      --  enumeration type.

      Thread_Enumerator_List : List_Id;
      --  Global list for all the distributed application threds
      --  enumeration types.

      Entity_Table_List : List_Id;
      --  Global list that associates each thread of the distributed
      --  application to its containing process. Concretly, this is a
      --  list of associations that contain for each enumerator of the
      --  Entity_Type type the corresponding enumerator of the
      --  Node_Type type.

      --  The currently handled process instance

      -----------------------
      -- Get_Node_Enum_Pos --
      -----------------------

      function Get_Node_Enum_Pos (Name : Name_Id) return Nat is
         I_Name   : Name_Id;
         Position : Nat;
      begin
         Set_Str_To_Name_Buffer ("%node_enum_pos%");
         Get_Name_String (Name);
         I_Name := Name_Find;

         --  Check whether a value has been computed for the current
         --  enumerator. Otherwise, compute a new one.

         Position := Get_Name_Table_Info (I_Name);

         if Position = 0 then
            --  Get the next free position

            Position := Last_Node_Enum_Pos;

            --  Link it to the enumerator for future use

            Set_Name_Table_Info (I_Name, Position);

            --  Increment the next free position

            Last_Node_Enum_Pos := Last_Node_Enum_Pos + 1;
         end if;

         return Position;
      end Get_Node_Enum_Pos;

      -------------------------
      -- Get_Thread_Enum_Pos --
      -------------------------

      function Get_Thread_Enum_Pos (Name : Name_Id) return Nat is
         I_Name   : Name_Id;
         Position : Nat;
      begin
         Set_Str_To_Name_Buffer ("%thread_enum_pos%");
         Get_Name_String (Name);
         I_Name := Name_Find;

         --  Check whether a value has been computed for the current
         --  enumerator. Otherwise, compute a new one.

         Position := Get_Name_Table_Info (I_Name);

         if Position = 0 then
            --  Get the next free position

            Position := Last_Thread_Enum_Pos;

            --  Link it to the enumerator for future use

            Set_Name_Table_Info (I_Name, Position);

            --  Increment the next free position

            Last_Thread_Enum_Pos := Last_Thread_Enum_Pos + 1;
         end if;

         return Position;
      end Get_Thread_Enum_Pos;

      -------------------------
      -- Insert_Node_In_List --
      -------------------------

      procedure Insert_Node_In_List
        (N            : Node_Id;
         L            : List_Id;
         Get_Enum_Pos : Get_Enum_Pos_Access)
      is
         use CTN;

         M          : Node_Id;
         Position_N : Nat;
         Position_M : Nat;
      begin
         pragma Assert (Kind (N) = K_Defining_Identifier);

         case CTN.Kind (N) is
            when CTN.K_Defining_Identifier =>
               Position_N := Get_Enum_Pos (CTN.Name (N));
            when others =>
               raise Program_Error with "Inconsistency in Insert_Node_In_List";
         end case;

         if Is_Empty (L) then
            Append_Node_To_List (N, L);
         else
            M := CTN.First_Node (L);

            while Present (M) loop
               case CTN.Kind (M) is
                  when CTN.K_Defining_Identifier =>
                     Position_M := Get_Enum_Pos (CTN.Name (M));
                  when others =>
                     raise Program_Error with
                       "Inconsistency in Insert_Node_In_List";
               end case;

               if Position_N < Position_M then
                  Insert_Before_Node (N, M, L);

                  return; --  IMPORTANT
               end if;

               M := CTN.Next_Node (M);
            end loop;

            Append_Node_To_List (N, L);
         end if;

      end Insert_Node_In_List;

      -----------
      -- Visit --
      -----------

      procedure Visit (E : Node_Id) is
      begin
         case Kind (E) is
            when K_Architecture_Instance =>
               Visit_Architecture_Instance (E);

            when K_Component_Instance =>
               Visit_Component_Instance (E);

            when others =>
               null;
         end case;
      end Visit;

      ---------------------------------
      -- Visit_Architecture_Instance --
      ---------------------------------

      procedure Visit_Architecture_Instance (E : Node_Id) is
      begin
         Visit (Root_System (E));
      end Visit_Architecture_Instance;

      ------------------------------
      -- Visit_Component_Instance --
      ------------------------------

      procedure Visit_Component_Instance (E : Node_Id) is
         Cathegory : constant Component_Category
           := Get_Category_Of_Component (E);
      begin
         case Cathegory is
            when CC_System =>
               Visit_System_Instance (E);

            when CC_Process =>
               Visit_Process_Instance (E);

            when CC_Thread =>
               Visit_Thread_Instance (E);

            when others =>
               null;
         end case;
      end Visit_Component_Instance;

      ----------------------------
      -- Visit_Process_Instance --
      ----------------------------

      procedure Visit_Process_Instance (E : Node_Id) is
         U        : constant Node_Id := CTN.Distributed_Application_Unit
           (CTN.Naming_Node (Backend_Node (Identifier (E))));
         P        : constant Node_Id := CTN.Entity (U);
         S        : constant Node_Id := Parent_Subcomponent (E);
         N        : Node_Id;
         C        : Node_Id;
      begin
         Push_Entity (P);
         Push_Entity (U);
         Set_Deployment_Header;

         --  Create the lists

         Node_Enumerator_List       := New_List (CTN.K_Enumeration_Literals);
         Node_Enumerator_Pos_List   := New_List (CTN.K_Element_List);
         Thread_Enumerator_List     := New_List (CTN.K_Enumeration_Literals);
         Entity_Table_List          := New_List (CTN.K_Element_List);

         Nb_Tasks := 0;

         --  Build the enumerator corresponding to the distributed
         --  application node. The enumerator name is mapped from the
         --  process *sub*component instance.

         N := Make_Defining_Identifier (Map_C_Enumerator_Name (S));
         Append_Node_To_List (N, Node_Enumerator_List);
         Bind_AADL_To_Enumerator (Identifier (S), N);

         --  Build the representation clause of the enumerator

         Insert_Node_In_List
           (N, Node_Enumerator_Pos_List, Get_Node_Enum_Pos'Access);

         --  Visit all the subcomponents of the process

         if not AAU.Is_Empty (Subcomponents (E)) then
            C := First_Node (Subcomponents (E));

            while Present (C) loop
               Visit (Corresponding_Instance (C));

               C := Next_Node (C);
            end loop;
         end if;

         --  Create the node enumeration type declaration. Note that
         --  the type creation is possible even the enumeration list
         --  is incomplete. We can do this in the first traversal
         --  since we are sure that the enumerator list is not empty.

         N := Message_Comment ("For each node in the distributed"
                               & " application add an enumerator");
         Append_Node_To_List (N, CTN.Declarations (Current_File));

         N := Make_Full_Type_Declaration
           (Defining_Identifier => RE (RE_Node_T),
            Type_Definition     => Make_Enum_Aggregate
              (Node_Enumerator_List));
         Append_Node_To_List (N, CTN.Declarations (Current_File));

         --  Create the thread enumeration type declaration. Note that
         --  the type creation is possible even the enumeration list
         --  is incomplete. This type may not be generated in case the
         --  application is local.

         if not Is_Empty (Thread_Enumerator_List) then
            N := Message_Comment ("For each thread in the distributed"
                                  & " application nodes, add an"
                                  & " enumerator");
            Append_Node_To_List (N, CTN.Declarations (Current_File));

            N := Make_Full_Type_Declaration
              (Defining_Identifier => RE (RE_Task_Id),
               Type_Definition     => Make_Enum_Aggregate
                 (Thread_Enumerator_List));
            Append_Node_To_List (N, CTN.Declarations (Current_File));
         end if;

         N := Make_Define_Statement
           (Defining_Identifier => RE (RE_Nb_Tasks),
             Value => Make_Literal (New_Int_Value (Nb_Tasks, 1, 10)));
         Append_Node_To_List (N, CTN.Declarations (Current_File));

         --  FIXME : When distributed application will be
         --  generated, we will have to declare
         --  the entity table list to know all the nodes
         --  For the moment, it is not necessary

--         if not Is_Empty (Entity_Table_List) then
--            N := Message_Comment ("Entity Table");
--            Append_Node_To_List (N, CTN.Declarations (Current_File));
--
--            N := Make_Array_Type_Definition
--              (Range_Constraints    => Make_List_Id
--                 (Make_Range_Constraint
--                  (No_Node,
--                   No_Node,
--                   Make_Attribute_Designator
--                   (Make_Designator
--                    (TN (T_Entity_Type)), A_Range))),
--               Component_Definition => Make_Defining_Identifier
--                 (TN (T_Node_Type)));
--
--            N := Make_Object_Declaration
--              (Defining_Identifier => Make_Defining_Identifier
--                 (PN (P_Entity_Table)),
--               Constant_Present    => True,
--               Object_Definition   => N,
--               Expression          => Make_Array_Aggregate
--                 (Entity_Table_List));
--            Append_Node_To_List (N, CTN.Declarations (Current_File));
--         end if;

         Pop_Entity; -- U
         Pop_Entity; -- P
      end Visit_Process_Instance;

      ---------------------------
      -- Visit_System_Instance --
      ---------------------------

      procedure Visit_System_Instance (E : Node_Id) is
         S : Node_Id;
      begin
         Push_Entity (HI_Distributed_Application_Root);

         --  Visit all the subcomponents of the system

         if not AAU.Is_Empty (Subcomponents (E)) then
            S := First_Node (Subcomponents (E));
            while Present (S) loop
               --  Visit the component instance corresponding to the
               --  subcomponent S.

               Visit (Corresponding_Instance (S));
               S := Next_Node (S);
            end loop;
         end if;

         Pop_Entity; --  HI_Distributed_Application_Root
      end Visit_System_Instance;

      ---------------------------
      -- Visit_Thread_Instance --
      ---------------------------

      procedure Visit_Thread_Instance (E : Node_Id) is
         N               : Node_Id;
         P               : Node_Id;
         S               : constant Node_Id := Parent_Subcomponent (E);
      begin

         Nb_Tasks := Nb_Tasks + 1;

         --  Build the enumerator corresponding to the thread. The
         --  enumerator name is mapped from the the thread name and
         --  its containing process name.

         N := Make_Defining_Identifier (Map_C_Enumerator_Name (S));
         Insert_Node_In_List
           (N,
            Thread_Enumerator_List,
            Get_Thread_Enum_Pos'Access);
         Bind_AADL_To_Enumerator (Identifier (S), N);

         --  Get the Process parent of the thread

         P := Parent_Component (S);
         pragma Assert (Is_Process (P));

         Insert_Node_In_List
           (N,
            Entity_Table_List,
            Get_Thread_Enum_Pos'Access);

      end Visit_Thread_Instance;

   end Header_File;

end Ocarina.Generators.PO_HI_C.Deployment;
