------------------------------------------------------------------------------
--                                                                          --
--                           OCARINA COMPONENTS                             --
--                                                                          --
--   O C A R I N A . G E N E R A T O R S . P O _ H I _ C . R U N T I M E    --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                 Copyright (C) 2007, GET-Telecom Paris.                   --
--                                                                          --
-- Ocarina  is free software;  you  can  redistribute  it and/or  modify    --
-- it under terms of the GNU General Public License as published by the     --
-- Free Software Foundation; either version 2, or (at your option) any      --
-- later version. Ocarina is distributed  in  the  hope  that it will be    --
-- useful, but WITHOUT ANY WARRANTY;  without even the implied warranty of  --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General --
-- Public License for more details. You should have received  a copy of the --
-- GNU General Public License distributed with Ocarina; see file COPYING.   --
-- If not, write to the Free Software Foundation, 51 Franklin Street, Fifth --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable to be   --
-- covered  by the  GNU  General  Public  License. This exception does not  --
-- however invalidate  any other reasons why the executable file might be   --
-- covered by the GNU Public License.                                       --
--                                                                          --
--                 Ocarina is maintained by the Ocarina team                --
--                       (ocarina-users@listes.enst.fr)                     --
--                                                                          --
------------------------------------------------------------------------------

package Ocarina.Generators.PO_HI_C.Runtime is

   --  Headers file that can be included

   type RH_Id is
     (RH_Null,                       --  Workaround to denote a null RH
      RH_PO_HI_Task,                 --  po_hi_task header
      RH_PO_HI_Time,                 --  po_hi_time header
      RH_PO_HI_Common,               --  po_hi_common header
      RH_Activity,                   --  po_hi_common header
      RH_Deployment
     );

   --  Runtime Entities

   type RE_Id is
     (RE_Null,                       --  Workaround to denote a null RE
      RE_Microseconds,               --  __po_hi_microseconds
      RE_Milliseconds,               --  __po_hi_milliseconds
      RE_Seconds,                    --  __po_hi_seconds
      RE_Minutes,                    --  __po_hi_minutes
      RE_Create_Periodic_Task,       -- __po_hi_create_periodic_task
      RE_Create_Sporadic_Task,       -- __po_hi_create_sporadic_task
      RE_Wait_For_Next_Period,       -- __po_hi_task_wait_for_next_period
      RE_Initialize_Tasking,         -- __po_hi_initialize_tasking
      RE_Sporadic_Wait,              -- __po_hi_sporadic_wait
      RE_Sporadic_Wait_Release,      -- __po_hi_sporadic_wait_release
      RE_Wait_For_Tasks,             -- __po_hi_wait_for_tasks

      RE_Default_Priority,           --  __PO_HI_DEFAULT_PRIORITY
      RE_Main_Type,                  --  __PO_HI_MAIN_TYPE
      RE_Main_Args,                  --  __PO_HI_MAIN_ARGS
      RE_Main_Name,                  --  __PO_HI_MAIN_NAME
      RE_Nb_Tasks,                   --  __PO_HI_NB_TASKS

      RE_Task_Id,                    --  __po_hi_task_id
      RE_Node_T                      --  __po_hi_node_t
     );

   --  Runtime types

   subtype RT_Id is RE_Id range RE_Task_Id .. RE_Node_T;
   subtype RC_Id is RE_Id range RE_Default_Priority .. RE_Nb_Tasks;
   subtype RF_Id is RE_Id range RE_Null .. RE_Wait_For_Tasks;

   RE_Header_Table : constant array (RE_Id) of RH_Id
     := (RE_Null                       => RH_Null,
         --  Runtime functions associations
         RE_Microseconds               => RH_PO_HI_Time,
         RE_Milliseconds               => RH_PO_HI_Time,
         RE_Seconds                    => RH_PO_HI_Time,
         RE_Minutes                    => RH_PO_HI_Time,
         RE_Create_Periodic_Task       => RH_PO_HI_Task,
         RE_Create_Sporadic_Task       => RH_PO_HI_Task,
         RE_Wait_For_Next_Period       => RH_PO_HI_Task,
         RE_Initialize_Tasking         => RH_PO_HI_Task,
         RE_Sporadic_Wait              => RH_PO_HI_Task,
         RE_Sporadic_Wait_Release      => RH_PO_HI_Task,
         RE_Wait_For_Tasks             => RH_PO_HI_Task,
         --  Runtime types associations
         RE_Task_Id                    => RH_Deployment,
         RE_Node_T                     => RH_Deployment,
         --  Runtime Constants associations
         RE_Default_Priority           => RH_PO_HI_Task,
         RE_Main_Type                  => RH_PO_HI_Common,
         RE_Main_Args                  => RH_PO_HI_Common,
         RE_Main_Name                  => RH_PO_HI_Common,
         RE_Nb_Tasks                   => RH_Deployment
        );
   procedure Initialize;
   procedure Reset;

   function RE (Id : RE_Id) return Node_Id;
   --  Return a designator for entity Id

   function RH (Id : RH_Id) return Node_Id;
   --  Return a node for entity RH_Id

end Ocarina.Generators.PO_HI_C.Runtime;
