#! /bin/sh

# This script generates AST manipulation Ada code from a pseudo IDL
# description of the tree. If the first parameter is "hard", the
# script exits with an error code if an error occurs. If the first
# parameter is "soft", the script exits with an error code if and only
# if the pseudo IDL file exists and contains errors.

usage () {
    cat << EOF
Usage: $0 (hard|soft) <Pseudo_IDL_File>
   hard: If the Ada files cannot be generated, exit with an error code
   soft: Do no exist with error code only if the pseudo IDL file exists
         and contains errors.
EOF
    exit 1
}

if test $# -ne 2 ; then
    usage;
fi;

# Get absolute paths for the script and the argument

current_dir=`pwd`

script_dir="`dirname $0`"; cd ${script_dir}; script_dir=`pwd` 
cd ${current_dir}

idl_tree_dir="`dirname $2`"; cd ${idl_tree_dir}; idl_tree_dir=`pwd`
cd ${current_dir}

idl_file="`basename $2`"
tree_ada_spec="`basename $2 .idl`.ads"
tree_ada_body="`basename $2 .idl`.adb"

tmp=nodes.tmp
tmp_dir=temp.$$
mknodes="${script_dir}/mknodes/mknodes"
update_headers="${script_dir}/../support/headers_ocarina"

# Tests whether mknodes exists

if test -x ${mknodes} ; then
    mknodes_exists=yes
else
    mknodes_exists=no
fi;

# Tests whether the pseudo IDL file exists

if test -f "${idl_tree_dir}/${idl_file}" ; then
    idl_file_exists=yes
else
    idl_file_exists=no
fi;

# Tests whether update_headers exists

# Compile ${update_headers} if possible

uheaders_dir="`dirname ${update_headers}`"
uheaders_src="${update_headers}.adb"

if test -d ${uheaders_dir} && \
    test -r ${uheaders_dir}/Makefile && \
    test -r ${uheaders_src} ; then
    make -C ${uheaders_dir} "`basename ${update_headers}`"
fi

if test -x ${update_headers} ; then
    update_headers_exists=yes
else
    update_headers_exists=no
fi;

# Test if all is OK

ALL_OK=yes

(test "${mknodes_exists}" = "yes")        || ALL_OK=no
(test "${idl_file_exists}" = "yes")       || ALL_OK=no

if test x${ALL_OK} = xno ; then
    if test x$1 = xhard ; then
	if test x${mknodes_exists} = xno ; then
	    echo "mknodes : not found"
	fi

	if test x${idl_file_exists} = xno ; then
	    echo "${idl_file} : not found"
	fi
	  
	# Exit with an error status

	exit 1;
    else
	# Exit silently

	exit 0;
    fi
fi

# Check whether ${update_headers} exists

if test x${update_headers_exists} = xno ; then
    if test x$1 = xhard ; then
	echo "${update_headers} not found"

        # Exit with an error status
	
	exit 1;
    fi
fi

# Process the pseudo IDL file in a temporary directory

echo "Processing ${idl_file}"
cd ${idl_tree_dir} 

mkdir ${tmp_dir} || exit $?

${mknodes} ${idl_file} > ${tmp}; 
EXIT_CODE=$?

gnatchop -w ${tmp} ${tmp_dir}; rm -f ${tmp}

# Update the copyright headers (if possible)

# FIXME This is temporarily disabled due to file encoding problem on
# DOS-like systems.

# if test x${update_headers_exists} = xyes ; then
#     ${update_headers} ${tmp_dir}/${tree_ada_spec}
#     ${update_headers} ${tmp_dir}/${tree_ada_body}
# fi

# Replace the tree files if they are different from the original files

for file in ${tmp_dir}/${tree_ada_spec} ${tmp_dir}/${tree_ada_body} ; do
    ofile="`basename ${file}`"
    if test -r ${ofile}  ; then
	if cmp ${file} ${ofile} > /dev/null; then
	    rm -f ${file}
	else
	    mv -f ${file} ${ofile}
	fi
    else
	mv -f ${file} ${ofile}
    fi
done

rm -rf ${tmp_dir}

cd ${current_dir}
exit ${EXIT_CODE}
